/*
 * Expectation-Maximisation Algorithm for the Mixture of Gaussians (MOG) model.
 *
 * This is only an API proposition and subjected to further changes.
 */

#ifndef _EM_MOG_H
#define _EM_MOG_H

struct em_mog_context;

struct em_mog_context *em_mog_new();
void em_mog_free(struct em_mog_context **ctx);
void em_mog_init(struct em_mog_context *ctx);
void em_mog_clear(struct em_mog_context *ctx);

/* Set number of gaussians in mixture. */
void em_mog_set_ngaussians(struct em_mog_context *ctx, int ngauss);
/* Get number of gaussians in mixture. */
int em_mog_get_ngaussians(struct em_mog_context *ctx);

/* Set fitted data item dimension. */
void em_mog_set_dimension(struct em_mog_context *ctx, int dim);
/* Get fitted data item dimension. */
int em_mog_get_dimension(struct em_mog_context *ctx);

/* Set n-th gaussian component weight. */
void em_mog_set_nth_weight(struct em_mog_context *ctx, int nth, double weight);
/* Get n-th gaussian component weight. */
double em_mog_get_nth_weight(struct em_mog_context *ctx, int nth);

/* Set n-th gaussian mean. */
void em_mog_set_nth_mean(struct em_mog_context *ctx, int nth, double mu)
/* Get n-th gaussian mean. */
double em_mog_get_nth_mean(struct em_mog_context *ctx, int nth)

/* 
 * Convariance matrix data goes in row-major order (C-style). This is a square
 * matrix of MxM size, where M is the dimension of data item. Dimension is
 * provided by context and not required as an argument.
 * 
 * / a b c \
 * | d e f | = { a, b, c, d, e, f, g, h, i }.
 * \ g h i /
 *
 */

/* Set n-th gaussian covariance matrix. */
void em_mog_set_nth_covariance_matrix(struct em_mog_context *ctx, int nth, const double *cmx);
/* Get n-th gaussian covariance matrix. */
void em_mog_get_nth_covariance_matrix(struct em_mog_context *ctx, int nth, double *cmx);

/* 
 * Do one step of EM-algorithm, returning logarithm of likelihood function.
 * Data fitted to the MOG model is provided by x and it's size in nx. It's
 * dimension is provided by context and not required as a separate argument.
 */
double em_mog_step(struct em_mog_context *ctx, double *x, int nx);

/*
 * Converge the EM-algorithm, repeating EM-steps until logarithm of the
 * likelihood function stops to change. Threshold of the change is given by epsilon.
 */
int em_mog_converge(struct em_mog_context *ctx, double *x, int nx, double epsilon);

#endif

